GENERAL_KEYBINDS = [
    ("ctrl+c", "Quit App"),
    ("ctrl+s", "Open Settings"),
]

TYPING_KEYBINDS = [
    ("escape", "Reset typing"),
    ("ctrl+n", "Toggle numbers"),
    ("ctrl+p", "Toggle punctuations"),
    ("ctrl+w", "Delete a whole word"),
    ("ctrl+l", "Opens language menu"),
    ("ctrl+t", "Opens theme menu"),
]

SETTINGS_KEYBINDS = [
    ("j/down", "Move to next setting"),
    ("k/up", "Move to previous setting"),
    ("tab", "Move to next option"),
    ("shift+tab", "Move to previous option"),
]
